from .gpt35 import GPTLogProb
from .llama2 import Llama2PPL
from .rwkv import RWKVPPL
from .mamba import MambaPPL
from .mistral import MixtralPPL
from .core import MLMPPL, DecoderPPL, EncoderDecoderPPL, LLMPPL




import torch
import math
from transformers import AutoTokenizer, AutoModelForCausalLM

class RWKVPPL:
    def __init__(self, model_name="RWKV/rwkv-raven-7b", device=None):
        """
        Initialize RWKV model for PPL calculation.
        
        Args:
            model_name (str): Name of the model on HuggingFace
            device (str): Device to use ('cuda' or 'cpu')
        """
        self.device = device if device else ('cuda' if torch.cuda.is_available() else 'cpu')
        print(f"Using device: {self.device}")

        print("Loading tokenizer...")
        self.tokenizer = AutoTokenizer.from_pretrained(model_name)
        
        print("Loading model...")
        self.model = AutoModelForCausalLM.from_pretrained(
            model_name,
            trust_remote_code=True,
            torch_dtype=torch.float16 if self.device == 'cuda' else torch.float32
        ).to(self.device)
        
        self.model.eval()
        print("Model loaded successfully")

    def calculate_ppl(self, text: str) -> float:
        """
        Calculate perplexity for the given text.
        
        Args:
            text (str): Input text
            
        Returns:
            float: Perplexity score
        """
        try:
            # Tokenize text
            encodings = self.tokenizer(text, return_tensors='pt', add_special_tokens=True).to(self.device)
            
            # Calculate loss
            with torch.no_grad():
                outputs = self.model(**encodings, labels=encodings["input_ids"])
                loss = outputs.loss.item()
            
            # Calculate perplexity
            ppl = math.exp(loss)
            return ppl
            
        except Exception as e:
            print(f"Error calculating perplexity: {e}")
            raise

def main():
    try:
        print("Initializing RWKV model...")

        # Use the RWKV/rwkv-raven-7b model
        rwkv_ppl = RWKVPPL(model_name="RWKV/rwkv-raven-7b")

        texts = [
            "The quick brown fox jumps over the lazy dog.",
            "Natural language processing is a subfield of artificial intelligence and linguistics.",
            "RWKV is a linear attention model that combines the best of RNN and Transformer.",
        ]

        print("\nCalculating perplexity for test texts:")
        for text in texts:
            try:
                ppl = rwkv_ppl.calculate_ppl(text)
                print(f"\nText: {text}")
                print(f"Perplexity: {ppl:.4f}")
            except Exception as e:
                print(f"Error processing text: {e}")

    except Exception as e:
        print(f"An error occurred: {e}")
        print("Please ensure you have enough GPU memory and all required packages are installed.")

if __name__ == "__main__":
    main()



import torch
from transformers import LlamaTokenizer, LlamaForCausalLM, BitsAndBytesConfig
import math
from huggingface_hub import HfFolder
import os
import importlib.metadata as metadata  # 替代 pkg_resources

def check_dependencies():
    required = {'torch', 'transformers', 'protobuf', 'bitsandbytes', 'accelerate'}
    installed = {pkg for pkg in metadata.distributions()}
    installed_names = {pkg.metadata['Name'] for pkg in installed}
    missing = required - installed_names

    if missing:
        raise ImportError(f"Missing required packages: {', '.join(missing)}. Please install them using pip.")

class Llama2PPL:
    def __init__(self, model_name="meta-llama/Llama-2-7b-hf", device=None):
        self.device = device if device else ('cuda' if torch.cuda.is_available() else 'cpu')
        print(f"Using device: {self.device}")

        token = HfFolder.get_token() or os.getenv('HUGGING_FACE_TOKEN')
        if not token:
            raise ValueError("No Hugging Face token found. Please log in using `huggingface-cli login` or set the HUGGING_FACE_TOKEN environment variable.")

        print("Loading tokenizer...")
        self.tokenizer = LlamaTokenizer.from_pretrained(model_name, token=token)
        
        print("Loading model...")
        quantization_config = BitsAndBytesConfig(
            load_in_8bit=True,
            llm_int8_threshold=6.0,
            llm_int8_has_fp16_weight=False
        )
        self.model = LlamaForCausalLM.from_pretrained(
            model_name,
            device_map="auto",
            quantization_config=quantization_config,
            token=token
        )
        self.model.eval()

        # Check for older GPU drivers
        if torch.cuda.is_available() and torch.cuda.get_device_properties(0).major >= 8:
            print("Note: If you're using an RTX 4000 series GPU, please ensure your GPU drivers are up to date to avoid P2P issues.")

    def calculate_ppl(self, text):
        encodings = self.tokenizer(text, return_tensors="pt")
        input_ids = encodings.input_ids.to(self.device)
        target_ids = input_ids.clone()

        with torch.no_grad():
            outputs = self.model(input_ids, labels=target_ids)
            
        loss = outputs.loss
        ppl = math.exp(loss.item())
        return ppl

def main():
    try:
        check_dependencies()
        llama_ppl = Llama2PPL()

        texts = [
            "The quick brown fox jumps over the lazy dog.",
            "Natural language processing is a subfield of artificial intelligence and linguistics.",
            "Llama 2 is a large language model developed by Meta AI.",
            "It's a point he's made many times before, and it's one that he'"
        ]

        for text in texts:
            ppl = llama_ppl.calculate_ppl(text)
            print(f"Text: {text}")
            print(f"Perplexity: {ppl:.4f}\n")

    except Exception as e:
        print(f"An error occurred: {e}")

if __name__ == "__main__":
    main()



import os
import logging
from time import sleep
import numpy as np
import tiktoken
import openai
from typing import List, Tuple, Optional

class GPTLogProb:
    """Language Model with encoding handling for log probability calculation."""

    def __init__(self, model: str, sleep_time: int = 10):
        logging.basicConfig(level=logging.INFO)
        logging.info(f'Initializing model: `{model}`')
        
        self.api_key = os.getenv('OPENAI_API_KEY')
        if not self.api_key:
            raise ValueError("OPENAI_API_KEY environment variable not set")
        
        openai.api_key = self.api_key
        self.model = model
        self.sleep_time = sleep_time
        self.encoding = tiktoken.get_encoding("cl100k_base")

    def get_logprobs(self, input_text: str) -> Tuple[List[List[Tuple[str, float]]], List[str]]:
        token_indices = self.encoding.encode(input_text)
        tokens = [self.encoding.decode([i]) for i in token_indices]

        all_logprobs = []

        for i in range(0, len(tokens)):
            current_prompt = "".join(tokens[:i])
            
            while True:
                try:
                    completion = openai.Completion.create(
                        engine=self.model,
                        prompt=current_prompt,
                        logprobs=10,
                        max_tokens=1,
                        temperature=0
                    )
                    break
                except openai.error.RateLimitError:
                    logging.info(f'Rate limit exceeded. Waiting for {self.sleep_time} seconds.')
                    sleep(self.sleep_time)
                except Exception as e:
                    logging.error(f'OpenAI API error: {e}')
                    raise

            if completion.choices[0].logprobs.top_logprobs:
                logprobs = completion.choices[0].logprobs.top_logprobs[0]
                all_logprobs.append(list(logprobs.items()))
            else:
                all_logprobs.append([])

        return all_logprobs, tokens

def calculate_ppl(input_text: str, model_name: str = "gpt-3.5-turbo-instruct") -> float:
    scorer = GPTLogProb(model=model_name)
    logprobs, tokens = scorer.get_logprobs(input_text)
    
    log_sum = 0.0
    token_count = 0

    print("Token-wise log probabilities:")
    for idx, token_set in enumerate(logprobs):
        actual_token = tokens[idx]
        token_dict = dict(token_set)
        
        if actual_token in token_dict:
            token_logprob = token_dict[actual_token]
            log_sum += token_logprob
            token_count += 1
            print(f"Token: '{actual_token}', LogProb: {token_logprob:.4f}")
        else:
            print(f"Token: '{actual_token}', LogProb: N/A (not in top 10 predictions)")
            logging.warning(f"Token '{actual_token}' not in top 10 predictions for position {idx}")

    if token_count == 0:
        raise ValueError("No valid tokens found for perplexity calculation")

    average_log_prob = log_sum / token_count
    perplexity = np.exp(-average_log_prob)
    
    # print(f"\nTotal tokens: {token_count}")
    # print(f"Average log probability: {average_log_prob:.4f}")
    
    return perplexity

if __name__ == "__main__":
    input_text = 'The quick brown fox jumps over the lazy dog.'
    
    perplexity = calculate_ppl(input_text)
    print(f"\nPerplexity: {perplexity:.4f}")


import llmppl

def test_mlm_ppl():
    """
    Test Masked Language Model Perplexity (MLM) using BERT model.
    """
    mlm = llmppl.MLMPPL(model_name='bert-base-uncased')
    text = "The quick brown fox jumps over the lazy dog."
    ppl = mlm.calculate_ppl(text)
    assert ppl > 0


def test_decoder_ppl():
    """
    Test Decoder Perplexity (Causal LM) using GPT-2 model.
    """
    decoder = llmppl.DecoderPPL(model_name='gpt2')
    text = "The quick brown fox jumps over the lazy dog."
    ppl = decoder.calculate_ppl(text)
    assert ppl > 0


def test_encoder_decoder_ppl():
    """
    Test Encoder-Decoder Perplexity using T5 model.
    """
    enc_dec = llmppl.EncoderDecoderPPL(model_name='t5-small')
    input_text = "translate English to German: The quick brown fox jumps over the lazy dog."
    output_text = "Der schnelle braune Fuchs springt über den faulen Hund."
    ppl = enc_dec.calculate_ppl(input_text, output_text)
    assert ppl > 0


def test_llmppl_mlm():
    """
    Test LLMPPL static method for MLM model.
    """
    text = "The quick brown fox jumps over the lazy dog."
    ppl = llmppl.LLMPPL.get_perplexity(text, model_type='mlm')
    assert ppl > 0


def test_llmppl_decoder():
    """
    Test LLMPPL static method for Decoder model.
    """
    text = "The quick brown fox jumps over the lazy dog."
    ppl = llmppl.LLMPPL.get_perplexity(text, model_type='clm', model_name='gpt2')
    assert ppl > 0


def test_llmppl_encoder_decoder():
    """
    Test LLMPPL static method for Encoder-Decoder model.
    """
    input_text = "translate English to German: The quick brown fox jumps over the lazy dog."
    output_text = "Der schnelle braune Fuchs springt über den faulen Hund."
    ppl = llmppl.LLMPPL.get_perplexity(input_text, output_text, model_type='enc-dec', model_name='t5-small')
    assert ppl > 0

def test_gpt35_logprob():
    model = llmppl.GPTLogProb(model='gpt-3.5-turbo-instruct')
    text = "The quick brown fox jumps over the lazy dog."
    logprobs, tokens = model.get_logprobs(text)
    assert len(logprobs) > 0
    assert len(tokens) > 0

def test_llama2_ppl():
    llama2 = llmppl.Llama2PPL()
    text = "The quick brown fox jumps over the lazy dog."
    ppl = llama2.calculate_ppl(text)
    assert ppl > 0

def test_rwkv_ppl():
    rwkv = llmppl.RWKVPPL()
    text = "The quick brown fox jumps over the lazy dog."
    ppl = rwkv.calculate_ppl(text)
    assert ppl > 0

def test_mamba_ppl():
    mamba = llmppl.MambaPPL()
    text = "The quick brown fox jumps over the lazy dog."
    ppl = mamba.calculate_ppl(text)
    assert ppl > 0

def test_mixtral_ppl():
    mixtral = llmppl.MixtralPPL()
    text = "The quick brown fox jumps over the lazy dog."
    ppl = mixtral.calculate_ppl(text)
    assert ppl > 0



import torch
from transformers import AutoModelForCausalLM, AutoTokenizer
import math
from typing import Union, List

class MambaPPL:
    def __init__(self, model_name: str = "state-spaces/mamba-130m-hf", device: str = None):
        """
        Initialize Mamba model for perplexity calculation.
        
        Args:
            model_name (str): Name of the model on HuggingFace
            device (str): Device to use ('cuda' or 'cpu')
        """
        self.device = device if device else ('cuda' if torch.cuda.is_available() else 'cpu')
        print(f"Using device: {self.device}")

        print("Loading tokenizer...")
        self.tokenizer = AutoTokenizer.from_pretrained(model_name)
        
        print("Loading model...")
        self.model = AutoModelForCausalLM.from_pretrained(
            model_name,
            trust_remote_code=True,
            torch_dtype=torch.float16 if self.device == 'cuda' else torch.float32
        ).to(self.device)
        
        self.model.eval()
        print(f"Model loaded successfully: {model_name}")

    def calculate_ppl(self, text: Union[str, List[str]], verbose: bool = False) -> Union[float, List[float]]:
        """
        Calculate perplexity for given text(s).
        
        Args:
            text: Input text or list of texts
            verbose: Whether to print token-level information
            
        Returns:
            Perplexity score(s)
        """
        if isinstance(text, str):
            return self._calculate_single_ppl(text, verbose)
        return [self._calculate_single_ppl(t, verbose) for t in text]

    def _calculate_single_ppl(self, text: str, verbose: bool = False) -> float:
        """Calculate perplexity for a single text."""
        try:
            # Tokenize text
            encodings = self.tokenizer(text, return_tensors='pt').to(self.device)
            input_ids = encodings.input_ids
            
            # Calculate loss
            with torch.no_grad():
                outputs = self.model(input_ids, labels=input_ids)
                loss = outputs.loss.item()

            if verbose:
                tokens = self.tokenizer.convert_ids_to_tokens(input_ids[0])
                print(f"\nTokens: {tokens}")
                print(f"Number of tokens: {len(tokens)}")
                print(f"Loss: {loss:.4f}")
            
            # Calculate perplexity
            ppl = math.exp(loss)
            return ppl
            
        except Exception as e:
            print(f"Error calculating perplexity: {e}")
            raise

def main():
    try:
        # Available model sizes
        MODEL_SIZES = {
            'tiny': "state-spaces/mamba-130m-hf",
            'small': "state-spaces/mamba-370m-hf",
            'base': "state-spaces/mamba-790m-hf",
            'large': "state-spaces/mamba-2.8b-hf"
        }

        # Initialize model (using smallest variant)
        print("\nInitializing Mamba model...")
        mamba_ppl = MambaPPL(model_name=MODEL_SIZES['tiny'])

        # Test texts
        texts = [
            "The quick brown fox jumps over the lazy dog.",
            "Natural language processing is a subfield of artificial intelligence and linguistics.",
            "Mamba is a state space model that provides an efficient alternative to Transformers.",
            # Add more test texts here
        ]

        # Individual text processing with verbose output
        print("\nProcessing individual texts:")
        for text in texts:
            ppl = mamba_ppl.calculate_ppl(text, verbose=True)
            print(f"Text: {text}")
            print(f"Perplexity: {ppl:.4f}\n")

        # Batch processing example
        print("\nBatch processing example:")
        ppls = mamba_ppl.calculate_ppl(texts)
        for text, ppl in zip(texts, ppls):
            print(f"Text: {text[:50]}...")
            print(f"Perplexity: {ppl:.4f}\n")

    except Exception as e:
        print(f"An error occurred: {e}")
        import traceback
        traceback.print_exc()

if __name__ == "__main__":
    main()


import torch
from transformers import AutoTokenizer, AutoModelForMaskedLM, AutoModelForCausalLM, AutoModelForSeq2SeqLM
import math
from tqdm import tqdm
import numpy as np

class MLMPPL:
    def __init__(self, model_name='bert-base-uncased', device=None):
        self.device = device if device else ('cuda' if torch.cuda.is_available() else 'cpu')
        self.tokenizer = AutoTokenizer.from_pretrained(model_name)
        self.model = AutoModelForMaskedLM.from_pretrained(model_name).to(self.device)
        self.model.eval()

    def calculate_ppl(self, texts, batch_size=8):
        if isinstance(texts, str):
            texts = [texts]

        nll = []
        for text in tqdm(texts, desc="Processing texts"):
            encodings = self.tokenizer(text, return_tensors='pt', padding=True, truncation=True)
            input_ids = encodings.input_ids.to(self.device)
            attention_mask = encodings.attention_mask.to(self.device)

            nll_sum = 0
            token_count = 0

            for i in range(input_ids.size(1)):
                masked_input_ids = input_ids.clone()
                masked_input_ids[:, i] = self.tokenizer.mask_token_id

                with torch.no_grad():
                    outputs = self.model(masked_input_ids, attention_mask=attention_mask)
                    logits = outputs.logits

                probs = torch.softmax(logits, dim=-1)
                true_token_probs = probs[0, i, input_ids[0, i]]
                nll_sum -= torch.log(true_token_probs).item()
                token_count += 1

            nll.append(nll_sum / token_count)

        ppl = [math.exp(x) for x in nll]
        return ppl if len(ppl) > 1 else ppl[0]

class DecoderPPL:
    def __init__(self, model_name='gpt2', device=None):
        self.device = device if device else ('cuda' if torch.cuda.is_available() else 'cpu')
        self.model = AutoModelForCausalLM.from_pretrained(model_name).to(self.device)
        self.tokenizer = AutoTokenizer.from_pretrained(model_name)
        if self.tokenizer.pad_token is None:
            self.tokenizer.pad_token = self.tokenizer.eos_token
        self.model.eval()

    def calculate_ppl(self, texts):
        if isinstance(texts, str):
            texts = [texts]

        ppls = []
        for text in tqdm(texts, desc="Processing texts"):
            encodings = self.tokenizer(text, return_tensors='pt', padding=True, truncation=True)
            max_length = self.model.config.max_position_embeddings
            stride = 512
            seq_len = encodings.input_ids.size(1)

            nlls = []
            prev_end_loc = 0
            for begin_loc in range(0, seq_len, stride):
                end_loc = min(begin_loc + max_length, seq_len)
                trg_len = end_loc - prev_end_loc  # may be different from stride on last loop
                input_ids = encodings.input_ids[:, begin_loc:end_loc].to(self.device)
                target_ids = input_ids.clone()
                target_ids[:, :-trg_len] = -100

                with torch.no_grad():
                    outputs = self.model(input_ids, labels=target_ids)
                    neg_log_likelihood = outputs.loss * trg_len

                nlls.append(neg_log_likelihood)

                prev_end_loc = end_loc
                if end_loc == seq_len:
                    break

            ppl = torch.exp(torch.stack(nlls).sum() / end_loc)
            ppls.append(ppl.item())

        return ppls if len(ppls) > 1 else ppls[0]

class EncoderDecoderPPL:
    def __init__(self, model_name='t5-small', device=None):
        self.device = device if device else ('cuda' if torch.cuda.is_available() else 'cpu')
        self.tokenizer = AutoTokenizer.from_pretrained(model_name)
        self.model = AutoModelForSeq2SeqLM.from_pretrained(model_name).to(self.device)
        self.model.eval()

    def calculate_ppl(self, input_texts, output_texts):
        if isinstance(input_texts, str):
            input_texts = [input_texts]
            output_texts = [output_texts]
        
        ppls = []
        for input_text, output_text in tqdm(zip(input_texts, output_texts), desc="Processing texts"):
            inputs = self.tokenizer(input_text, return_tensors="pt").to(self.device)
            with torch.no_grad():
                outputs = self.model(**inputs, labels=self.tokenizer(output_text, return_tensors="pt")["input_ids"].to(self.device))
            
            loss = outputs.loss
            ppl = math.exp(loss.item())
            ppls.append(ppl)

        return ppls if len(ppls) > 1 else ppls[0]

class LLMPPL:
    @staticmethod
    def get_perplexity(input_text, output_text=None, model_name='bert-base-uncased', model_type='mlm'):
        if model_type == 'mlm':
            calculator = MLMPPL(model_name)
            return calculator.calculate_ppl(input_text)
        elif model_type == 'clm':
            calculator = DecoderPPL(model_name)
            return calculator.calculate_ppl(input_text)
        elif model_type == 'enc-dec':
            if output_text is None:
                raise ValueError("output_text must be provided for encoder-decoder models")
            calculator = EncoderDecoderPPL(model_name)
            return calculator.calculate_ppl(input_text, output_text)
        else:
            raise ValueError("model_type must be 'mlm', 'clm', or 'enc-dec'")

# Example usage
if __name__ == "__main__":
    texts = [
        "Natural language processing is a subfield of linguistics and artificial intelligence.",
        "The quick brown fox jumps over the lazy dog."
    ]

    print("\nTesting Encoder-Decoder models:")
    enc_dec_models = ['t5-small', 'google/flan-t5-small']
    input_texts = [
        "translate English to German: The quick brown fox jumps over the lazy dog.",
        "summarize: Natural language processing is a subfield of linguistics and artificial intelligence."
    ]
    output_texts = [
        "Der schnelle braune Fuchs springt über den faulen Hund.",
        "NLP is a field combining linguistics and AI."
    ]
    for model in enc_dec_models:
        print(f"\nCalculating perplexity using {model}:")
        ppls = LLMPPL.get_perplexity(input_texts, output_texts, model, 'enc-dec')
        for input_text, output_text, ppl in zip(input_texts, output_texts, ppls):
            print(f"Input: {input_text}")
            print(f"Output: {output_text}")
            print(f"Perplexity: {ppl:.4f}\n")

    # Example with a longer text
    longer_input = """
    Translate the following text to French:
    Natural language processing (NLP) is a subfield of linguistics, computer science, and artificial intelligence 
    concerned with the interactions between computers and human language, in particular how to program computers to 
    process and analyze large amounts of natural language data.
    """
    longer_output = """
    Le traitement du langage naturel (TLN) est un sous-domaine de la linguistique, de l'informatique et de l'intelligence artificielle
    qui s'intéresse aux interactions entre les ordinateurs et le langage humain, en particulier à la manière de programmer les ordinateurs pour
    traiter et analyser de grandes quantités de données en langage naturel.
    """
    print("\nLonger input text:")
    print(longer_input)
    print("\nExpected output:")
    print(longer_output)
    print("\nCalculating perplexity using T5:")
    ppl_longer = LLMPPL.get_perplexity(longer_input, longer_output, 't5-small', 'enc-dec')
    print(f"Perplexity: {ppl_longer:.4f}")


import torch
from transformers import AutoTokenizer, AutoModelForCausalLM, BitsAndBytesConfig
import math

class MixtralPPL:
    def __init__(self, model_name="mistralai/Mistral-7B-v0.1"):
        self.device = 'cuda' if torch.cuda.is_available() else 'cpu'
        self.tokenizer = AutoTokenizer.from_pretrained(model_name)
        
        quantization_config = BitsAndBytesConfig(
            load_in_4bit=True,
            bnb_4bit_compute_dtype=torch.float16,
            bnb_4bit_use_double_quant=True,
            bnb_4bit_quant_type="nf4"
        )
        
        primary_gpu = 0 if self.device == 'cuda' else None
        max_memory = {
            0: "16GB",
            "cpu": "48GB"
        } if primary_gpu is not None else None
        
        self.model = AutoModelForCausalLM.from_pretrained(
            model_name,
            quantization_config=quantization_config,
            device_map="sequential",
            max_memory=max_memory,
            trust_remote_code=True,
            torch_dtype=torch.float16,
            offload_folder="offload_folder",
        )
        
        self.model.eval()

    def calculate_ppl(self, text: str) -> float:
        try:
            inputs = self.tokenizer(
                text,
                return_tensors="pt",
                truncation=True,
                max_length=256
            )
            input_ids = inputs["input_ids"]
            
            with torch.no_grad():
                outputs = self.model(input_ids, labels=input_ids)
                ppl = math.exp(outputs.loss.item())
            
            torch.cuda.empty_cache()
            return ppl
            
        except Exception as e:
            return float('inf')

def calculate_ppl(text: str, model_name: str = "mistralai/Mathstral-7b-v0.1") -> float:
    model = MixtralPPL(model_name)
    return model.calculate_ppl(text)

if __name__ == "__main__":
    texts = [
        "The quick brown fox jumps over the lazy dog.",
        "Natural language processing is a subfield of artificial intelligence and linguistics.",
        "Mixtral is a large language model developed by Mistral AI using sparse mixture of experts.",
        "The concept of perplexity in language models quantifies how well a probability model predicts a sample."
    ]

    model = MixtralPPL()
    for text in texts:
        ppl = model.calculate_ppl(text)
        print(f"Perplexity: {ppl:.4f}")


from setuptools import setup, find_packages

setup(
    name="llmppl",
    version="0.1.1",
    packages=find_packages(),
    install_requires=[
        "torch==2.5.0",
        "transformers==4.45.2",
        "openai==0.28.0",
        "tqdm==4.66.5",
        "sentencepiece==0.2.0",
        "bitsandbytes==0.44.1",
        "accelerate==1.0.1",
        "protobuf==5.28.2",
        "tiktoken==0.8.0",
    ],
    author="Zhenyu Xu",
    author_email="thornscrown1220@gmail.com",
    description="A package for calculating perplexity using various language models",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Arrtourz/llmppl",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7,<3.11',
)